@@ -1,4 +1,4 @@
-#!/usr/bin/wish
+#!/usr/bin/wish8.3
 #############################################################################
 # Visual Tcl v1.20 Project
 #
@@ -20,6 +20,7 @@ global widget;
     set widget(ModemMenu) {.top17.cpd21.03.04}
     set widget(faxnumlabel) {.top17.fra29.lab35}
     set widget(memcanvas) {.top17.fra29.can18}
+    set widget(printdialog) {.top21}
     set widget(progresscanvas) {.top17.fra29.can20}
     set widget(radio4rings) {.top17.cpd21.03.04.men36}
     set widget(rev,.top17.cpd21.03.04) {ModemMenu}
@@ -31,6 +32,7 @@ global widget;
     set widget(rev,.top17.fra29.can20) {progresscanvas}
     set widget(rev,.top17.fra29.lab33) {voicenumlabel}
     set widget(rev,.top17.fra29.lab35) {faxnumlabel}
+    set widget(rev,.top21) {printdialog}
     set widget(stopbutton) {.top17.fra22.but23}
     set widget(text) {.top17.fra24.tex25}
     set widget(voicenumlabel) {.top17.fra29.lab33}
@@ -40,9 +42,9 @@ global widget;
 #
 proc init {argc argv} {
 #
-# TkUsr v0.77
+# TkUsr v0.80
 #
-# Copyright (C) 1998-2001 Ludovic Drolez (ldrolez@free.fr)
+# Copyright (C) 1998-2003 Ludovic Drolez (ldrolez@free.fr)
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -59,8 +61,10 @@ proc init {argc argv} {
 #   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.       
 #
 # done:
+# fax printing
 # 
 # todo: 
+# save new, save all (with command line only option)
 # save the size of the GUI
 # progress for download
 # stop the building of stats with one click (for the impatient)
@@ -86,7 +90,7 @@ proc init {argc argv} {
     set device(lastreset) 0
     set device(auto) 0
     set device(binary) 0
-    set device(test) 0
+    set device(test) 1
     set device(lastvaliddate) [clock seconds]
     # AT command to go to Self Mode
     set device(MCS) "+MCS"
@@ -95,10 +99,15 @@ proc init {argc argv} {
     set stat(0.type) 2
     set stat(255.type) 2
 
-    set gui(version) 0.77
+    set gui(version) 0.80
     set gui(textback) lightgrey
     set gui(textfore) #A2FFAC
 
+    # 
+    set gui(print.to) "file"
+    set gui(print.file) "fax.ps"
+    set gui(print.command) "lpr -Plp"
+
     # variables which contain balloon help
     set gui(mem.balloon) ""
     set gui(voice.balloon) ""
@@ -209,7 +218,6 @@ proc init {argc argv} {
     option add *selectBackground	#c3c3c3
     option add *troughColor		#c3c3c3
     option add *disabledForeground	#a3a3a3
-    
 }
 
 init $argc $argv
@@ -279,17 +287,46 @@ proc {ByteUnstuff} {data array} {
 	return $numpage
 }
 
+proc {CheckLockedDevice} {dev} {
+#
+# Try to check if the serial port is locked by another process
+# returns: 1=locked   0=ok
+#
+    set locked 1
+
+    set fid [open $dev {RDWR NONBLOCK}]
+    fconfigure $fid -translation binary
+
+    set i 0
+    # the modem shound respond in less than 2 seconds
+    while { $i < 25 } {
+	if {($i == 0) || ($i == 10)} {
+	    puts -nonewline $fid "AT\r\r"
+	    flush $fid
+	}
+	set l [gets $fid]
+	if {[regexp OK $l]} {set locked 0;break}  
+	incr i
+	after 100
+    }
+    close $fid
+
+    return $locked
+}
+
 proc {ClearModem} {} {
 #
 # clear modem's memory
 #
-    global env widget
+    global env widget gui
     
+    set gui(status) "Clearing Memory..."
     # clear messages
     SendGetOK "AT+MEM\r"
     # reset the modem clock
     ModemClockReset
     # clear the text widget
+    set gui(status) ""
     UIReload
 }
 
@@ -356,6 +393,59 @@ global gui
     set gui(debug) $level
 }
 
+proc {FaxPrint} {w} {
+#
+# convert a fax to postscript
+#
+    global gui widget
+
+    set w $gui(print.canvas)
+
+    if {$gui(print.to) == "file"} {
+	$w postscript -file $gui(print.file) -pagex 0.i -pagey 11.i -pageanchor nw -pagewidth 8i -pageheight 11i
+    } else {
+	# must have at least tcl8.3 to print to a channel ?
+	set out [open "|$gui(print.command)" r+]
+	$w postscript -channel $out -pagex 0.i -pagey 11.i -pageanchor nw -pagewidth 8i -pageheight 11i
+	close $out
+    }
+    destroy $widget(printdialog)
+}
+
+proc {G3BuildCanvas} {{title {}}} {
+#
+# open a window and set up the canvas that will display the fax
+#
+    global gui
+
+    set w [toplevel .top[clock clicks]]
+    set c "$w.can"
+    set gui(print.canvas) $c
+
+    frame $w.grid
+    frame $w.grid.bottom -relief raised
+    scrollbar $w.hscroll -orient horiz -command "$c xview"
+    scrollbar $w.vscroll -command "$c yview"
+    canvas $c -relief sunken -borderwidth 2 -scrollregion {0 0 1728 1159}  -height 1159 -width 864 -xscrollcommand "$w.hscroll set"  -yscrollcommand "$w.vscroll set"
+    button $w.lp -text Print -command {Window show $widget(printdialog)} -width 30
+    button $w.quit -text Close -command "destroy $w" -width 30
+    pack $w.grid -expand yes -fill both -padx 1 -pady 1
+    grid rowconfig    $w.grid 0 -weight 1 -minsize 0
+    grid columnconfig $w.grid 0 -weight 1 -minsize 0
+    grid $c -padx 1 -in $w.grid -pady 1  -row 0 -column 0 -rowspan 1 -columnspan 1 -sticky news
+    grid $w.vscroll -in $w.grid -padx 1 -pady 1  -row 0 -column 1 -rowspan 1 -columnspan 1 -sticky news
+    grid $w.hscroll -in $w.grid -padx 1 -pady 1  -row 1 -column 0 -rowspan 1 -columnspan 1 -sticky news
+    grid $w.grid.bottom -in $w.grid -padx 1 -pady 1  -row 2 -column 0 -rowspan 1 -columnspan 2 -sticky ew
+
+    grid $w.lp -in $w.grid.bottom -padx 1 -pady 1  -row 0 -column 0 -rowspan 1 -columnspan 1 -sticky news
+    grid $w.quit -in $w.grid.bottom -padx 1 -pady 1  -row 0 -column 1 -rowspan 1 -columnspan 1 -sticky news
+
+    wm title $w $title
+    
+    tkwait visibility $w
+    return $c
+}
+
 proc {G3CheckBit} {bitref binstream} {
 upvar $bitref bit
     set i [string index $binstream $bit]
@@ -375,52 +465,118 @@ global heap heaplist
     }
 }
 
-proc {G3LoadAndDisplay} { {mode ""} } {
-    #
-    set name [tk_getOpenFile -defaultextension g3 ]
-    if {$name == ""} return
-    set fd [open $name]
-
-    set c [G3BuildCanvas $name]
-    if {$mode == "view2" } {
-	G3DisplayFDdiv2 $fd  $c
-    } else {
-	G3DisplayFD $fd  $c
-    }
-    close $fd
-}
-
-proc {G3BuildCanvas} { {title ""}} {
-#
-# open a window and set up the canvas that will display the fax
-#
-    set w [toplevel .top[clock clicks]]
-    set c "$w.can"
-    frame $w.grid
-    scrollbar $w.hscroll -orient horiz -command "$c xview"
-    scrollbar $w.vscroll -command "$c yview"
-    canvas $c -relief sunken -borderwidth 2 -scrollregion {0 0 1728 1159}  -height 1159 -width 864 -xscrollcommand "$w.hscroll set"  -yscrollcommand "$w.vscroll set"
-    button $w.quit -text close -command "destroy $w"
-    pack $w.grid -expand yes -fill both -padx 1 -pady 1
-    grid rowconfig    $w.grid 0 -weight 1 -minsize 0
-    grid columnconfig $w.grid 0 -weight 1 -minsize 0
-    grid $c -padx 1 -in $w.grid -pady 1  -row 0 -column 0 -rowspan 1 -columnspan 1 -sticky news
-    grid $w.vscroll -in $w.grid -padx 1 -pady 1  -row 0 -column 1 -rowspan 1 -columnspan 1 -sticky news
-    grid $w.hscroll -in $w.grid -padx 1 -pady 1  -row 1 -column 0 -rowspan 1 -columnspan 1 -sticky news
-    grid $w.quit -in $w.grid -padx 1 -pady 1  -row 2 -column 0 -rowspan 1 -columnspan 2 -sticky news
-
-    wm title $w $title
+proc {G3Display} {stream canvas} {
+## 1st version: directly translated from c (PIII 600MHz): 104 sec
+## optimized version: 7.5 sec
+## with creation of ppm files: 8 sec (no display) 9.5 (display)
+## width/2: 10.1sec (no disp) 11.5 sec (disp)
+    global heap
     
-    tkwait visibility $w
-    return $c
-}
+    #
+    set o ""
+    
+    set w 1728
+    set bandh 256
+    set h 2560
+    set count 0
+    
+    # 
+    set pix(0) ""
+    set pix(1) ""
+    for {set i 0} {$i <= $w} {incr i} {
+	append pix(0) "\xFF"
+	append pix(1) "\x0"
+    }
+    # read the file and convert it to binary string
+    binary scan $stream B* binstream
+    set len [string length $binstream]
+    set bit 0
+    
+    while {[G3CheckBit bit $binstream]} { }
+    
+    set linecnt 0
+    set eolcnt 0
+    
+    while {$eolcnt < 5 && $linecnt < 2560} {
+	
+	while {![G3CheckBit bit $binstream]} {}
+	set linelen 0
+	set j 0
+	
+	set outbuf ""	
+	while {1} {
+	    set i $heap(0,$j)
+	    while {$i > 0} {
+		# checkbit inlined
+		set i $heap($i,[string index $binstream $bit])
+		incr bit
+	    }
+	    
+	    if {$len <= $bit || $i == -32768}  break
+	    set i [string trimleft $i -]
+	    set k $i
+	    
+	    if {$k + $linelen > 1728} {
+		set k [expr 1728 - $linelen]
+	    }
+	    # put ($k) white or black dots 
+	    append outbuf [string range $pix($j) 1 $k]
+	    incr linelen $k
+	    if {$i < 64} {
+		if {$j} {
+		    set j 0
+		} else {
+		    set j 1
+		}
+	    }
+	}
+	if {$linelen == 0} {
+	    incr eolcnt
+	} else {
+	    set eolcnt 0
+	}
+	
+	# output 
+	append o $outbuf
+	# 
+	if {$linelen < $w} {
+	    append o [string range $pix(1) 1 [expr $w - $linelen]]
+	}
 
-proc {G3DisplayFDdiv2} {fd canvas} {
-    fconfigure $fd -translation binary
-    set stream [read $fd]
-    return [G3DisplayDiv2 $stream $canvas]
-}
+	incr linecnt
+	
+	# show progress
+	if {($linecnt % 40) == 0} {
+	    set f [open "/tmp/tkusr.pgm" w]
+	    puts $f "P5"
+	    puts $f "$w 40"
+	    puts $f "255"
+	    puts $f $o
+	    close $f
 
+	    #
+	    # exec cat /tmp/x.pgm | pgmtopbm | pbmtoxbm > /tmp/x.xbm
+	    #
+	    set im [image create photo]
+	    $im read /tmp/tkusr.pgm
+#	    set im [image create bitmap -file /tmp/x.xbm ]
+#	    set im2 [image create photo]
+#	    $im2 copy $im -subsample 2 2
+	    $canvas create image 0 $linecnt -image $im -anchor sw
+	    update
+	    update idletasks
+	    set o ""
+	    file delete /tmp/tkusr.pgm
+	}
+	
+    }
+    if {$eolcnt} {
+	set linecnt [expr $linecnt - $eolcnt]
+    }    
+    set h $linecnt
+    
+    return $o
+}
 
 proc {G3DisplayDiv2} {stream canvas} {
 ## 1st version: directly translated from c (PIII 600MHz): 104 sec
@@ -511,7 +667,7 @@ proc {G3DisplayDiv2} {stream canvas} {
 	
 	# show progress
 	if {($linecnt % 40) == 0} {
-	    set f [open "/tmp/x.pgm" w]
+	    set f [open "/tmp/tkusr.pgm" w]
 	    puts $f "P5"
 	    puts $f "$w 40"
 	    puts $f "255"
@@ -522,7 +678,7 @@ proc {G3DisplayDiv2} {stream canvas} {
 	    # exec cat /tmp/x.pgm | pgmtopbm | pbmtoxbm > /tmp/x.xbm
 	    #
 	    set im [image create photo]
-	    $im read /tmp/x.pgm 
+	    $im read /tmp/tkusr.pgm 
 #	    set im [image create bitmap -file /tmp/x.xbm ]
 #	    set im2 [image create photo]
 #	    $im2 copy $im -subsample 2 2
@@ -530,6 +686,7 @@ proc {G3DisplayDiv2} {stream canvas} {
 	    update
 	    update idletasks
 	    set o ""
+	    file delete /tmp/tkusr.pgm
 	}
 	
     }
@@ -542,125 +699,33 @@ proc {G3DisplayDiv2} {stream canvas} {
 }
 
 proc {G3DisplayFD} {fd canvas} {
-    fconfigure $fd -translation binary
+fconfigure $fd -translation binary
     set stream [read $fd]
     return [G3Display $stream $canvas]
 }
 
-proc {G3Display} {stream canvas} {
-
-## 1st version: directly translated from c (PIII 600MHz): 104 sec
-## optimized version: 7.5 sec
-## with creation of ppm files: 8 sec (no display) 9.5 (display)
-## width/2: 10.1sec (no disp) 11.5 sec (disp)
-    global heap
-    
-    #
-    set o ""
-    
-    set w 1728
-    set bandh 256
-    set h 2560
-    set count 0
-    
-    # 
-    set pix(0) ""
-    set pix(1) ""
-    for {set i 0} {$i <= $w} {incr i} {
-	append pix(0) "\xFF"
-	append pix(1) "\x0"
-    }
-    # read the file and convert it to binary string
-    binary scan $stream B* binstream
-    set len [string length $binstream]
-    set bit 0
-    
-    while {[G3CheckBit bit $binstream]} { }
-    
-    set linecnt 0
-    set eolcnt 0
-    
-    while {$eolcnt < 5 && $linecnt < 2560} {
-	
-	while {![G3CheckBit bit $binstream]} {}
-	set linelen 0
-	set j 0
-	
-	set outbuf ""	
-	while {1} {
-	    set i $heap(0,$j)
-	    while {$i > 0} {
-		# checkbit inlined
-		set i $heap($i,[string index $binstream $bit])
-		incr bit
-	    }
-	    
-	    if {$len <= $bit || $i == -32768}  break
-	    set i [string trimleft $i -]
-	    set k $i
-	    
-	    if {$k + $linelen > 1728} {
-		set k [expr 1728 - $linelen]
-	    }
-	    # put ($k) white or black dots 
-	    append outbuf [string range $pix($j) 1 $k]
-	    incr linelen $k
-	    if {$i < 64} {
-		if {$j} {
-		    set j 0
-		} else {
-		    set j 1
-		}
-	    }
-	}
-	if {$linelen == 0} {
-	    incr eolcnt
-	} else {
-	    set eolcnt 0
-	}
-	
-	# output 
-	append o $outbuf
-	# 
-	if {$linelen < $w} {
-	    append o [string range $pix(1) 1 [expr $w - $linelen]]
-	}
-
-	incr linecnt
-	
-	# show progress
-	if {($linecnt % 40) == 0} {
-	    set f [open "/tmp/x.pgm" w]
-	    puts $f "P5"
-	    puts $f "$w 40"
-	    puts $f "255"
-	    puts $f $o
-	    close $f
-
-	    #
-	    # exec cat /tmp/x.pgm | pgmtopbm | pbmtoxbm > /tmp/x.xbm
-	    #
-	    set im [image create photo]
-	    $im read /tmp/x.pgm 
-#	    set im [image create bitmap -file /tmp/x.xbm ]
-#	    set im2 [image create photo]
-#	    $im2 copy $im -subsample 2 2
-	    $canvas create image 0 $linecnt -image $im -anchor sw
-	    update
-	    update idletasks
-	    set o ""
-	}
-	
-    }
-    if {$eolcnt} {
-	set linecnt [expr $linecnt - $eolcnt]
-    }    
-    set h $linecnt
-    
-    return $o
+proc {G3DisplayFDdiv2} {fd canvas} {
+fconfigure $fd -translation binary
+    set stream [read $fd]
+    return [G3DisplayDiv2 $stream $canvas]
 }
 
-proc {GetFax} {number {disp "save"} } {
+proc {G3LoadAndDisplay} {{mode {}}} {
+#
+    set name [tk_getOpenFile -defaultextension g3 ]
+    if {$name == ""} return
+    set fd [open $name]
+
+    set c [G3BuildCanvas $name]
+    if {$mode == "view2" } {
+	G3DisplayFDdiv2 $fd  $c
+    } else {
+	G3DisplayFD $fd  $c
+    }
+    close $fd
+}
+
+proc {GetFax} {number {disp save}} {
 #
 # retreive a fax in g3 format, and view or save it
 #
@@ -801,7 +866,7 @@ proc {GetMemPage} {page} {
 #
 # get a memory page and cache it for fast retrieving
 #
-    global device
+    global device gui
 
     set device(d) ""
     if {[info exists device(cache.$page)]} {
@@ -813,9 +878,11 @@ proc {GetMemPage} {page} {
     # get the page
     set device(data) 0
     Send "AT+MTP=$page\r"
-#Debug 2 "#"
+
     # wait for data 
+    set gui(status) "Getting data..."
     tkwait variable device(data)
+    set gui(status) ""
     set device(buffer) ""
 #Debug 2 "##"
     # cache the page
@@ -938,9 +1005,19 @@ proc {GetModemRetrievalPassword} {} {
 
 proc {GetModemStats} {num} {
 #
+# Return a list containing informations for message 'num'
 #
-#
-    return [split [lindex [SendGetOK "AT+MSR=$num\r"] 0] , ]
+    set l [split [lindex [SendGetOK "AT+MSR=$num\r"] 0] , ]
+
+    # check for garbage in the fax/phone id field
+    if {$num != 0} {
+	if { [string length [lindex $l 8]] != 20 } {
+	    # garbage !!
+	    Debug 2 "FaxId contains garbage !"
+	    set l "[lrange $l 0 8] [lrange $l [expr [llength $l]-4] end ]" 
+	}
+    }
+    return $l 
 }
 
 proc {GetModemVoiceDuration} {} {
@@ -955,33 +1032,6 @@ proc {GetModemVoiceDuration} {} {
     }
 }
 
-proc {CheckLockedDevice} {dev} {
-#
-# Try to check if the serial port is locked by another process
-# returns: 1=locked   0=ok
-#
-    set locked 1
-
-    set fid [open $dev {RDWR NONBLOCK}]
-    fconfigure $fid -translation binary
-
-    set i 0
-    # the modem shound respond in less than 2 seconds
-    while { $i < 25 } {
-	if {($i == 0) || ($i == 10)} {
-	    puts -nonewline $fid "AT\r\r"
-	    flush $fid
-	}
-	set l [gets $fid]
-	if {[regexp OK $l]} {set locked 0;break}  
-	incr i
-	after 100
-    }
-    close $fid
-
-    return $locked
-}
-
 proc {InitDevice} {dev} {
 #
 # open the device file and set callbacks
@@ -1127,9 +1177,14 @@ proc {OpenModem} {} {
     SendGetOK "$device(initstring)\r"
     # query the modem type
     set type [lindex [SendGetOK "ATI3\r"] 0]
+    # extract rom version
+    set ver ""
+    regexp {Rev. ([0-9]+\.[0-9]+)} $type trash ver
     if {[regexp -nocase pro $type]} {
 	# professional message modem
 	set device(MCS) "+MCA"
+    } elseif {$ver >= 14.8} {
+	set device(MCS) "+MCA"
     } else {
 	if {![regexp -nocase message $type]} {
 	    tk_messageBox -icon error -title "Unknown Modem" -type ok -message "
@@ -1288,14 +1343,22 @@ proc {SetModemFaxIDString} {id} {
 #
 # set the fax id string 
 #
+    global gui
+
+    set gui(status) "Setting ID..."
     SendGetOK "AT+MFI=\"[string range $id 0 19]\"\r"
+    set gui(status) ""
 }
 
 proc {SetModemRetrievalPassword} {pass} {
 #
 # set the retrieval password 
 #
+    global gui
+    
+    set gui(status) "Setting password..."
     SendGetOK "AT+MCP=[string range $pass 0 3]\r"
+    set gui(status) ""
 }
 
 proc {SetModemVoiceDuration} {d} {
@@ -1645,7 +1708,7 @@ proc vTclWindow.top17 {base} {
     toplevel $base -class Toplevel \
         -cursor hand2 
     wm focusmodel $base passive
-    wm geometry $base 446x227+162+180
+    wm geometry $base 446x231+269+205
     wm maxsize $base 1009 870
     wm minsize $base 1 1
     wm overrideredirect $base 0
@@ -1655,9 +1718,10 @@ proc vTclWindow.top17 {base} {
     frame $base.cpd21 \
         -borderwidth 2 -height 30 -relief raised -width 30 
     menubutton $base.cpd21.01 \
-        -anchor w -menu .top17.cpd21.01.02 -padx 4 -pady 3 -text File 
+        -activeforeground black -anchor w -foreground black \
+        -menu .top17.cpd21.01.02 -padx 4 -pady 3 -text File 
     menu $base.cpd21.01.02 \
-        -cursor {} 
+        -activeforeground black -cursor {} -foreground black 
     $base.cpd21.01.02 add command \
         -command {G3LoadAndDisplay view} -label {View G3 file} 
     $base.cpd21.01.02 add command \
@@ -1669,36 +1733,37 @@ proc vTclWindow.top17 {base} {
     $base.cpd21.01.02 add command \
         -command Quit -label Quit 
     menubutton $base.cpd21.05 \
-        -anchor w -menu .top17.cpd21.05.06 -padx 4 -pady 3 -text Help \
-        -width 4 
+        -activeforeground black -anchor w -foreground black \
+        -menu .top17.cpd21.05.06 -padx 4 -pady 3 -text Help -width 4 
     menu $base.cpd21.05.06 \
-        -tearoff 0 
+        -activeforeground black -foreground black -tearoff 0 
     $base.cpd21.05.06 add command \
         \
-        -command {tk_messageBox -type ok -message "TkUsr $gui(version)\n\n Copyright (C) 1998-2001 Ludovic Drolez <ldrolez@free.fr>\n\nLicensed under GNU GPL."} \
+        -command {tk_messageBox -type ok -message "TkUsr $gui(version)\n\n Copyright (C) 1998-2003 Ludovic Drolez <ldrolez@free.fr>\n\nLicensed under GNU GPL."} \
         -label About 
     menubutton $base.cpd21.03 \
-        -anchor w -menu .top17.cpd21.03.04 -padx 4 -pady 3 -text Modem 
+        -activeforeground black -anchor w -foreground black \
+        -menu .top17.cpd21.03.04 -padx 4 -pady 3 -text Modem 
     menu $base.cpd21.03.04 \
-        -cursor {} 
+        -activeforeground black -cursor {} -foreground black 
     $base.cpd21.03.04 add checkbutton \
-        -command {set device(auto) $Standalone} -label {Standalone mode} \
-        -variable Standalone 
+        -variable Standalone -command {set device(auto) $Standalone} \
+        -label {Standalone mode} 
     $base.cpd21.03.04 add checkbutton \
-        \
+        -variable FaxReception \
         -command {set device(faxreception) $FaxReception;set device(faxreception.changed) 1} \
-        -label {Fax reception} -variable FaxReception 
+        -label {Fax reception} 
     $base.cpd21.03.04 add checkbutton \
-        \
+        -variable VoiceReception \
         -command {set device(voicereception) $VoiceReception;set device(voicereception.changed) 1} \
-        -label {Voice reception} -variable VoiceReception 
+        -label {Voice reception} 
     $base.cpd21.03.04 add cascade \
-        -label {Answer After} -menu .top17.cpd21.03.04.men36 
+        -menu .top17.cpd21.03.04.men36 -label {Answer After} 
     $base.cpd21.03.04 add separator
     $base.cpd21.03.04 add checkbutton \
-        \
+        -variable DialupRetrieval \
         -command {set device(dialupretrieval) $DialupRetrieval;set device(dialupretrieval.changed) 1} \
-        -label {Dialup retrieval} -variable DialupRetrieval 
+        -label {Dialup retrieval} 
     $base.cpd21.03.04 add command \
         -command UISetPassword -label {Set Password} 
     $base.cpd21.03.04 add command \
@@ -1712,23 +1777,24 @@ proc vTclWindow.top17 {base} {
     $base.cpd21.03.04 add command \
         -command ClearModem -label {Clear memory} 
     menu $base.cpd21.03.04.men36 \
-        -tearoff 0 
+        -activeforeground black -foreground black -tearoff 0 
     $base.cpd21.03.04.men36 add radiobutton \
-        -command {set device(rings) 3} -label {3 rings} -value 3 \
-        -variable device(rings) 
+        -value 3 -variable device(rings) -command {set device(rings) 3} \
+        -label {3 rings} 
     $base.cpd21.03.04.men36 add radiobutton \
-        -command {set device(rings) 4} -label {4 rings} -value 4 \
-        -variable device(rings) 
+        -value 4 -variable device(rings) -command {set device(rings) 4} \
+        -label {4 rings} 
     $base.cpd21.03.04.men36 add radiobutton \
-        -command {set device(rings) 5} -label {5 rings} -value 5 \
-        -variable device(rings) 
+        -value 5 -variable device(rings) -command {set device(rings) 5} \
+        -label {5 rings} 
     $base.cpd21.03.04.men36 add radiobutton \
-        -command {set device(rings) 6} -label {6 rings} -value 6 \
-        -variable device(rings) 
+        -value 6 -variable device(rings) -command {set device(rings) 6} \
+        -label {6 rings} 
     menubutton $base.cpd21.men19 \
-        -menu .top17.cpd21.men19.m -padx 4 -pady 3 -text Messages 
+        -activeforeground black -foreground black -menu .top17.cpd21.men19.m \
+        -padx 4 -pady 3 -text Messages 
     menu $base.cpd21.men19.m \
-        -cursor {} 
+        -activeforeground black -cursor {} -foreground black 
     $base.cpd21.men19.m add command \
         -command {PlayMess 0} -label {Play Outgoing Message} 
     $base.cpd21.men19.m add command \
@@ -1745,16 +1811,19 @@ proc vTclWindow.top17 {base} {
     frame $base.fra22 \
         -borderwidth 2 -height 75 -width 125 
     button $base.fra22.but23 \
-        -borderwidth 1 \
+        -activeforeground black -borderwidth 1 \
         -command {Send " ";$widget(stopbutton) configure -state disabled} \
-        -image stop -padx 9 -pady 3 -state disabled 
+        -foreground black -image stop -padx 9 -pady 3 -state disabled 
     button $base.fra22.but28 \
-        -borderwidth 1 -image play -padx 9 -pady 3 -state disabled 
+        -activeforeground black -borderwidth 1 -foreground black -image play \
+        -padx 9 -pady 3 -state disabled 
     frame $base.fra24 \
         -borderwidth 2 -height 75 -relief groove -width 125 
     text $base.fra24.tex25 \
-        -background lightgrey -cursor hand2 -font { helvetica 12 } -height 10 \
-        -width 60 -wrap none -yscrollcommand {.top17.fra24.scr26 set} 
+        -background lightgrey -cursor hand2 -font { helvetica 12 } \
+        -foreground black -height 10 -insertbackground black \
+        -selectforeground #d9d9d9 -width 60 -wrap none \
+        -yscrollcommand {.top17.fra24.scr26 set} 
     bind $base.fra24.tex25 <Button-1> {
         PlayMess [expr int([lindex [$widget(text) dump -mark current] 2])]
     }
@@ -1762,29 +1831,37 @@ proc vTclWindow.top17 {base} {
         Popup [expr int([lindex [$widget(text) dump -mark current] 2])] %X %Y
     }
     scrollbar $base.fra24.scr26 \
-        -borderwidth 1 -command {.top17.fra24.tex25 yview} -orient vert \
-        -width 10 
+        -borderwidth 1 -command {.top17.fra24.tex25 yview} -width 10 
     frame $base.fra29 \
         -borderwidth 1 -height 75 -relief sunken -width 125 
     label $base.fra29.lab30 \
-        -borderwidth 1 -text Mem: 
+        -activeforeground black -borderwidth 1 -foreground black -text Mem: 
     label $base.fra29.lab32 \
-        -borderwidth 1 -text {   Voice:} 
+        -activeforeground black -borderwidth 1 -foreground black \
+        -text {   Voice:} 
     label $base.fra29.lab33 \
-        -borderwidth 1 -relief sunken -text 0 -textvariable gui(voice) 
+        -activeforeground black -borderwidth 1 -foreground black \
+        -relief sunken -text 0 -textvariable gui(voice) 
     label $base.fra29.lab34 \
-        -borderwidth 1 -text {   Fax:} 
+        -activeforeground black -borderwidth 1 -foreground black \
+        -text {   Fax:} 
     label $base.fra29.lab35 \
-        -borderwidth 1 -relief sunken -text 0 -textvariable gui(fax) 
+        -activeforeground black -borderwidth 1 -foreground black \
+        -relief sunken -text 0 -textvariable gui(fax) 
     label $base.fra29.lab18 \
-        -borderwidth 1 -textvariable gui(status) 
+        -activeforeground black -borderwidth 1 -foreground black \
+        -textvariable gui(status) 
     canvas $base.fra29.can18 \
-        -borderwidth 1 -height 14 -relief sunken -width 48 
+        -borderwidth 1 -height 14 -insertbackground black -relief sunken \
+        -selectforeground #d9d9d9 -width 48 
     canvas $base.fra29.can20 \
-        -borderwidth 1 -height 14 -relief raised -width 100 
+        -borderwidth 1 -height 14 -insertbackground black -relief raised \
+        -selectforeground #d9d9d9 -width 100 
     ###################
     # SETTING GEOMETRY
     ###################
+    grid columnconf $base 0 -weight 1
+    grid rowconf $base 3 -weight 1
     grid $base.cpd21 \
         -in .top17 -column 0 -row 1 -columnspan 1 -rowspan 1 -sticky ew 
     pack $base.cpd21.01 \
@@ -1804,9 +1881,12 @@ proc vTclWindow.top17 {base} {
         -in .top17.fra22 -column 1 -row 0 -columnspan 1 -rowspan 1 -ipadx 8 \
         -ipady 2 
     grid $base.fra24 \
-        -in .top17 -column 0 -row 3 -columnspan 1 -rowspan 1 -sticky news
+        -in .top17 -column 0 -row 3 -columnspan 1 -rowspan 1 -sticky nesw 
+    grid columnconf $base.fra24 0 -weight 1
+    grid rowconf $base.fra24 0 -weight 1
     grid $base.fra24.tex25 \
-        -in .top17.fra24 -column 0 -row 0 -columnspan 1 -rowspan 1 -sticky news
+        -in .top17.fra24 -column 0 -row 0 -columnspan 1 -rowspan 1 \
+        -sticky nesw 
     grid $base.fra24.scr26 \
         -in .top17.fra24 -column 1 -row 0 -columnspan 1 -rowspan 1 \
         -sticky nesw 
@@ -1843,23 +1923,36 @@ proc vTclWindow.top18 {base} {
     ###################
     toplevel $base -class Toplevel
     wm focusmodel $base passive
-    wm geometry $base 171x55+306+225
+    wm geometry $base 171x55+307+247
     wm maxsize $base 1009 870
     wm minsize $base 1 1
     wm overrideredirect $base 0
     wm resizable $base 1 1
     wm title $base "Set Retrieve Password"
-    label $base.lab19  -borderwidth 1 -text Password: 
-    entry $base.ent20  -textvariable device(retrievalpassword) -width 4 
-    button $base.but21   -command {SetModemRetrievalPassword $device(retrievalpassword);destroy .top18}  -padx 9 -pady 3 -text Set -width 8 
-    button $base.but22  -command {destroy .top18} -padx 9 -pady 3 -text Cancel -width 8 
+    label $base.lab19 \
+        -activeforeground black -borderwidth 1 -foreground black \
+        -text Password: 
+    entry $base.ent20 \
+        -foreground black -insertbackground black -selectforeground #d9d9d9 \
+        -textvariable device(retrievalpassword) -width 4 
+    button $base.but21 \
+        -activeforeground black \
+        -command {SetModemRetrievalPassword $device(retrievalpassword);destroy .top18} \
+        -foreground black -padx 9 -pady 3 -text Set -width 8 
+    button $base.but22 \
+        -activeforeground black -command {destroy .top18} -foreground black \
+        -padx 9 -pady 3 -text Cancel -width 8 
     ###################
     # SETTING GEOMETRY
     ###################
-    grid $base.lab19  -in .top18 -column 0 -row 0 -columnspan 1 -rowspan 1 -sticky e 
-    grid $base.ent20  -in .top18 -column 1 -row 0 -columnspan 1 -rowspan 1 -sticky w 
-    grid $base.but21  -in .top18 -column 0 -row 2 -columnspan 1 -rowspan 1 
-    grid $base.but22  -in .top18 -column 1 -row 2 -columnspan 1 -rowspan 1 -pady 4
+    grid $base.lab19 \
+        -in .top18 -column 0 -row 0 -columnspan 1 -rowspan 1 -sticky e 
+    grid $base.ent20 \
+        -in .top18 -column 1 -row 0 -columnspan 1 -rowspan 1 -sticky w 
+    grid $base.but21 \
+        -in .top18 -column 0 -row 2 -columnspan 1 -rowspan 1 
+    grid $base.but22 \
+        -in .top18 -column 1 -row 2 -columnspan 1 -rowspan 1 -pady 4 
 }
 
 proc vTclWindow.top19 {base} {
@@ -1874,23 +1967,36 @@ proc vTclWindow.top19 {base} {
     ###################
     toplevel $base -class Toplevel
     wm focusmodel $base passive
-    wm geometry $base 222x56+420+181
+    wm geometry $base 222x56+421+203
     wm maxsize $base 1009 870
     wm minsize $base 1 1
     wm overrideredirect $base 0
     wm resizable $base 1 1
     wm title $base "Set FaxID String"
-    label $base.lab19  -borderwidth 1 -text {Fax ID:} 
-    entry $base.ent20  -textvariable device(faxidstring) 
-    button $base.but21  -command {SetModemFaxIDString $device(faxidstring);destroy .top19}  -padx 9 -pady 3 -text Set -width 11 
-    button $base.but22  -command {destroy .top19} -padx 9 -pady 3 -text Cancel -width 11 
+    label $base.lab19 \
+        -activeforeground black -borderwidth 1 -foreground black \
+        -text {Fax ID:} 
+    entry $base.ent20 \
+        -foreground black -insertbackground black -selectforeground #d9d9d9 \
+        -textvariable device(faxidstring) 
+    button $base.but21 \
+        -activeforeground black \
+        -command {SetModemFaxIDString $device(faxidstring);destroy .top19} \
+        -foreground black -padx 9 -pady 3 -text Set -width 11 
+    button $base.but22 \
+        -activeforeground black -command {destroy .top19} -foreground black \
+        -padx 9 -pady 3 -text Cancel -width 11 
     ###################
     # SETTING GEOMETRY
     ###################
-    grid $base.lab19  -in .top19 -column 0 -row 0 -columnspan 2 -rowspan 1 -sticky w 
-    grid $base.ent20  -in .top19 -column 0 -row 0 -columnspan 2 -rowspan 1 -sticky e 
-    grid $base.but21  -in .top19 -column 0 -row 2 -columnspan 1 -rowspan 1 
-    grid $base.but22  -in .top19 -column 1 -row 2 -columnspan 1 -rowspan 1 -pady 4
+    grid $base.lab19 \
+        -in .top19 -column 0 -row 0 -columnspan 2 -rowspan 1 -sticky w 
+    grid $base.ent20 \
+        -in .top19 -column 0 -row 0 -columnspan 2 -rowspan 1 -sticky e 
+    grid $base.but21 \
+        -in .top19 -column 0 -row 2 -columnspan 1 -rowspan 1 
+    grid $base.but22 \
+        -in .top19 -column 1 -row 2 -columnspan 1 -rowspan 1 -pady 4 
 }
 
 proc vTclWindow.top20 {base} {
@@ -1905,23 +2011,111 @@ proc vTclWindow.top20 {base} {
     ###################
     toplevel $base -class Toplevel
     wm focusmodel $base passive
-    wm geometry $base 170x56+328+181
+    wm geometry $base 170x56+329+203
     wm maxsize $base 1009 870
     wm minsize $base 1 1
     wm overrideredirect $base 0
     wm resizable $base 1 1
     wm title $base "Set Max Voice Duration"
-    label $base.lab19  -borderwidth 1 -text {Duration (in sec):} 
-    entry $base.ent20  -textvariable device(voiceduration) -width 3 
-    button $base.but21   -command {SetModemVoiceDuration $device(voiceduration);destroy .top20}  -padx 9 -pady 3 -text Set -width 8 
-    button $base.but22  -command {destroy .top20} -padx 9 -pady 3 -text Cancel -width 8 
+    label $base.lab19 \
+        -activeforeground black -borderwidth 1 -foreground black \
+        -text {Duration (in sec):} 
+    entry $base.ent20 \
+        -foreground black -insertbackground black -selectforeground #d9d9d9 \
+        -textvariable device(voiceduration) -width 3 
+    button $base.but21 \
+        -activeforeground black \
+        -command {SetModemVoiceDuration $device(voiceduration);destroy .top20} \
+        -foreground black -padx 9 -pady 3 -text Set -width 8 
+    button $base.but22 \
+        -activeforeground black -command {destroy .top20} -foreground black \
+        -padx 9 -pady 3 -text Cancel -width 8 
     ###################
     # SETTING GEOMETRY
     ###################
-    grid $base.lab19  -in .top20 -column 0 -row 0 -columnspan 2 -rowspan 1 -sticky w 
-    grid $base.ent20  -in .top20 -column 1 -row 0 -columnspan 2 -rowspan 1 -sticky e 
-    grid $base.but21  -in .top20 -column 0 -row 2 -columnspan 1 -rowspan 1 
-    grid $base.but22  -in .top20 -column 1 -row 2 -columnspan 1 -rowspan 1 -pady 4
+    grid $base.lab19 \
+        -in .top20 -column 0 -row 0 -columnspan 2 -rowspan 1 -sticky w 
+    grid $base.ent20 \
+        -in .top20 -column 1 -row 0 -columnspan 2 -rowspan 1 -sticky e 
+    grid $base.but21 \
+        -in .top20 -column 0 -row 2 -columnspan 1 -rowspan 1 
+    grid $base.but22 \
+        -in .top20 -column 1 -row 2 -columnspan 1 -rowspan 1 -pady 4 
+}
+
+proc vTclWindow.top21 {base} {
+    if {$base == ""} {
+        set base .top21
+    }
+    if {[winfo exists $base]} {
+        wm deiconify $base; return
+    }
+    ###################
+    # CREATING WIDGETS
+    ###################
+    toplevel $base -class Toplevel
+    wm focusmodel $base passive
+    wm geometry $base 287x104+339+394
+    wm maxsize $base 1009 870
+    wm minsize $base 1 1
+    wm overrideredirect $base 0
+    wm resizable $base 1 1
+    wm title $base "Print to..."
+    frame $base.fra22 \
+        -borderwidth 2 -height 75 -relief groove -width 125 
+    label $base.fra22.lab23 \
+        -activeforeground black -borderwidth 1 -foreground black \
+        -text {Print command:} 
+    entry $base.fra22.ent24 \
+        -foreground black -insertbackground black -selectforeground #d9d9d9 \
+        -textvariable gui(print.command) -width 15 
+    label $base.fra22.lab25 \
+        -activeforeground black -anchor e -borderwidth 1 -foreground black \
+        -text File: 
+    entry $base.fra22.ent26 \
+        -foreground black -insertbackground black -selectforeground #d9d9d9 \
+        -textvariable gui(print.file) -width 15 
+    button $base.fra22.but27 \
+        -activeforeground black -command {FaxPrint $gui(print.canvas)} -foreground black \
+        -text Print 
+    button $base.fra22.but28 \
+        -activeforeground black -command {destroy $widget(printdialog)} \
+        -foreground black -text Cancel 
+    label $base.fra22.lab29 \
+        -activeforeground black -borderwidth 1 \
+        -font {Helvetica -12 {bold italic}} -foreground black \
+        -text {Print to:} 
+    radiobutton $base.fra22.rad30 \
+        -activeforeground black -foreground black -text printer \
+        -variable gui(print.to) -value "printer"
+    radiobutton $base.fra22.rad31 \
+        -activeforeground black -anchor e -foreground black -text file \
+        -variable gui(print.to) -value "file"
+    ###################
+    # SETTING GEOMETRY
+    ###################
+    grid $base.fra22 \
+        -in .top21 -column 0 -row 0 -columnspan 1 -rowspan 1 
+    grid columnconf $base.fra22 0 -minsize 140
+    grid columnconf $base.fra22 1 -minsize 140
+    grid $base.fra22.lab23 \
+        -in .top21.fra22 -column 0 -row 1 -columnspan 1 -rowspan 1 -sticky e 
+    grid $base.fra22.ent24 \
+        -in .top21.fra22 -column 1 -row 1 -columnspan 1 -rowspan 1 -sticky w 
+    grid $base.fra22.lab25 \
+        -in .top21.fra22 -column 0 -row 2 -columnspan 1 -rowspan 1 -sticky e 
+    grid $base.fra22.ent26 \
+        -in .top21.fra22 -column 1 -row 2 -columnspan 1 -rowspan 1 -sticky w 
+    grid $base.fra22.but27 \
+        -in .top21.fra22 -column 0 -row 4 -columnspan 1 -rowspan 1 -sticky ew 
+    grid $base.fra22.but28 \
+        -in .top21.fra22 -column 1 -row 4 -columnspan 1 -rowspan 1 -sticky ew 
+    grid $base.fra22.lab29 \
+        -in .top21.fra22 -column 0 -row 0 -columnspan 1 -rowspan 1 -sticky e 
+    grid $base.fra22.rad30 \
+        -in .top21.fra22 -column 1 -row 0 -columnspan 1 -rowspan 1 -sticky w 
+    grid $base.fra22.rad31 \
+        -in .top21.fra22 -column 1 -row 0 -columnspan 1 -rowspan 1 -sticky e 
 }
 
 Window show .
